module top(
        input               clk_in,
        input               rstn,

        output              led_d1,
        output              led_d2,
        output              led_d3,
        output              led_d4,

        input               uart_rx,
        output              uart_tx
    );

wire            rstn_ip_rstn;
wire            rstn_ip_uart;
wire            rstn_ip;

wire            pll_uart_lock;
reg     [15:0]  uart_delay_cnt = 16'd0;
reg             rstn_uart_d1 = 0;
reg             rstn_uart_d2 = 0;
wire            clk_uart;
reg             rstn_uart;

//UART PORT
wire    [15:0]  uart_addr;
wire            uart_wren;
wire            uart_wdat_rdy;
wire    [31:0]  uart_wdata;
wire            uart_rden;
wire            uart_rdat_vld;
wire    [31:0]  uart_rdata;

wire    [31:0]  reg0x0000;  wire    [31:0]  reg0x0010;  wire    [31:0]  reg0x0020;  wire    [31:0]  reg0x0030;
wire    [31:0]  reg0x0001;  wire    [31:0]  reg0x0011;  wire    [31:0]  reg0x0021;
wire    [31:0]  reg0x0002;  wire    [31:0]  reg0x0012;  wire    [31:0]  reg0x0022;

wire    [31:0]  mon0x0080;  wire    [31:0]  mon0x0090;  wire    [31:0]  mon0x00A0;  wire    [31:0]  mon0x00B0;
wire    [31:0]  mon0x0081;  wire    [31:0]  mon0x0091;  wire    [31:0]  mon0x00A1;  wire    [31:0]  mon0x00B1;
                                                        wire    [31:0]  mon0x00A2;  wire    [31:0]  mon0x00B2;
                                                        wire    [31:0]  mon0x00A3;  wire    [31:0]  mon0x00B3;
//IP configuration_vector
wire    [1*4-1:0]   sgmii_mode;
wire    [9*4-1:0]   sgmii_link_timer;

wire    [1*4-1:0]   main_reset;
wire    [1*4-1:0]   an_enable;
wire    [1*4-1:0]   an_restart;
wire    [3*4-1:0]   confi_vector;
wire    [1*4-1:0]   disparity_checking_en;

reg     [1*4-1:0]   ip_link_complete;
reg     [1*4-1:0]   an_complete;
reg     [1*4-1:0]   an_complete_d1;
reg     [8*4-1:0]   ip_link_complete_cnt_tx;
reg     [1*4-1:0]   tx_mac_rst_n;
reg     [8*4-1:0]   ip_link_complete_cnt_rx;
reg     [1*4-1:0]   rx_mac_rst_n;
reg     [1*4-1:0]   auto_negotiation_complete;

reg     [1*4-1:0]   cycle_one_time;
reg     [1*4-1:0]   cycle_all_time;

//IP TX PORT
wire    [1*4-1:0]   tx_mac_clk/* synthesis syn_keep=1 */;
wire    [1*4-1:0]   tx_mac_valid;
wire    [8*4-1:0]   tx_mac_data;
wire    [1*4-1:0]   tx_mac_last;
wire    [1*4-1:0]   tx_mac_error;
wire    [1*4-1:0]   tx_mac_ready;

wire    [1*4-1:0]   tx_pause_req;
wire    [16*4-1:0]  tx_pause_val;
wire    [48*4-1:0]  tx_pause_source_addr;
wire    [1*4-1:0]   tx_statistics_valid;

//IP RX PORT
wire    [1*4-1:0]   rx_mac_clk/* synthesis syn_keep=1 */;
wire    [1*4-1:0]   rx_mac_valid;
wire    [8*4-1:0]   rx_mac_data;
wire    [1*4-1:0]   rx_mac_last;
wire    [1*4-1:0]   rx_mac_error;

wire    [1*4-1:0]   rx_statistics_valid;
wire    [27*4-1:0]  rx_statistics_vector;
wire    [1*4-1:0]   rx_pause_req;
wire    [16*4-1:0]  rx_pause_val;

//IP PORT
reg     [15*4-1:0]  configuration_vector;
wire    [11*4-1:0]  status_vector;

wire    [1*4-1:0]   tx_running;
wire    [1*4-1:0]   rx_running;
reg     [1*4-1:0]   rx_running_d1;
wire    [32*4-1:0]  tx_cnt;
wire    [32*4-1:0]  rx_cnt;

button sw1(
        .clk    (clk_in         ),
        .bin    (rstn           ),
        .bout   (rstn_ip_rstn   )
    );

button uart1(
        .clk    (clk_in         ),
        .bin    (reg0x0030[0]   ),
        .bout   (rstn_ip_uart   )
    );

assign  rstn_ip =   rstn_ip_rstn && rstn_ip_uart;

pll_uart u_pll_uart(
        .clkout0(clk_uart       ), //output clkout
        .lock   (pll_uart_lock  ), //output lock
        .reset  (~rstn          ), //input reset
        .clkin  (clk_in         ) //input clkin
    );

always @(posedge clk_uart) begin
    if(~pll_uart_lock) begin
        uart_delay_cnt  <=  16'd0;
    end else begin
        if(uart_delay_cnt == 16'hffff) begin
            uart_delay_cnt  <=  uart_delay_cnt;
        end else begin
            uart_delay_cnt  <=  uart_delay_cnt + 16'd1;
        end
    end
end

always @(posedge clk_uart) begin
    rstn_uart_d1    <=  uart_delay_cnt[15];
    rstn_uart_d2    <=  rstn_uart_d1;
    rstn_uart       <=  rstn_uart_d2;
end

SerDes_Top u_SerDes_Top(
        .QSGMII_Top_clk_in_i                        (clk_in                         ),
        .QSGMII_Top_rstn_i                          (rstn_ip                        ),

    //connected to an optical module to detect the presence of light
        .QSGMII_Top_signal_detect_ch0_i             (1'b1                           ),//1:presence; 0:not presence
    //config port
    //link_timer; multiplied by 32.768us
        .QSGMII_Top_sgmii_link_timer_ch0_i          (sgmii_link_timer[8:0]          ),
        .QSGMII_Top_sgmii_mode_ch0_i                (sgmii_mode[0]                  ),//0:mac mode; 1:phy mode
        .QSGMII_Top_repeater_mode_ch0_i             (1'b0                           ),

        .QSGMII_Top_rx_mac_clk_ch0_o                (rx_mac_clk[0]                  ),
        .QSGMII_Top_rx_mac_valid_ch0_o              (rx_mac_valid[0]                ),
        .QSGMII_Top_rx_mac_data_ch0_o               (rx_mac_data[0*8+:8]            ),
        .QSGMII_Top_rx_mac_last_ch0_o               (rx_mac_last[0]                 ),
        .QSGMII_Top_rx_mac_error_ch0_o              (rx_mac_error[0]                ),
        
        .QSGMII_Top_tx_mac_clk_ch0_o                (tx_mac_clk[0]                  ),
        .QSGMII_Top_tx_mac_valid_ch0_i              (tx_mac_valid[0]                ),
        .QSGMII_Top_tx_mac_data_ch0_i               (tx_mac_data[0*8+:8]            ),
        .QSGMII_Top_tx_mac_last_ch0_i               (tx_mac_last[0]                 ),
        .QSGMII_Top_tx_mac_error_ch0_i              (tx_mac_error[0]                ),
        .QSGMII_Top_tx_mac_ready_ch0_o              (tx_mac_ready[0]                ),
        .QSGMII_Top_tx_collision_ch0_o              (                               ),
        .QSGMII_Top_tx_retransmit_ch0_o             (                               ),

        .QSGMII_Top_rx_fcs_fwd_ena_ch0_i            (1'b0                           ),
        .QSGMII_Top_rx_jumbo_ena_ch0_i              (1'b0                           ),
        .QSGMII_Top_rx_statistics_valid_ch0_o       (rx_statistics_valid[0]         ),
        .QSGMII_Top_rx_statistics_vector_ch0_o      (rx_statistics_vector[0*27+:27] ),
        .QSGMII_Top_rx_pause_req_ch0_o              (rx_pause_req[0]                ),
        .QSGMII_Top_rx_pause_val_ch0_o              (rx_pause_val[0*16+:16]         ),

        .QSGMII_Top_tx_pause_req_ch0_i              (tx_pause_req[0]                ),
        .QSGMII_Top_tx_pause_val_ch0_i              (tx_pause_val[0*16+:16]         ),
        .QSGMII_Top_tx_pause_source_addr_ch0_i      (tx_pause_source_addr[0*48+:48] ),
        .QSGMII_Top_tx_ifg_delay_ena_ch0_i          (1'b0                           ),
        .QSGMII_Top_tx_ifg_delay_ch0_i              (8'd0                           ),
        .QSGMII_Top_tx_fcs_fwd_ena_ch0_i            (1'b0                           ),
        .QSGMII_Top_tx_statistics_valid_ch0_o       (tx_statistics_valid[0]         ),
        .QSGMII_Top_tx_statistics_vector_ch0_o      (                               ),
    //mdio
        .QSGMII_Top_miim_hs_clk_ch0_i               (1'b0                           ),
        .QSGMII_Top_miim_phyad_ch0_i                ({5{1'b0}}                      ),
        .QSGMII_Top_miim_regad_ch0_i                ({5{1'b0}}                      ),
        .QSGMII_Top_miim_wrdata_ch0_i               ({16{1'b0}}                     ),
        .QSGMII_Top_miim_wren_ch0_i                 (1'b0                           ),
        .QSGMII_Top_miim_rden_ch0_i                 (1'b0                           ),
        .QSGMII_Top_miim_rddata_ch0_o               (                               ),
        .QSGMII_Top_miim_rddata_valid_ch0_o         (                               ),
        .QSGMII_Top_miim_busy_ch0_o                 (                               ),
        .QSGMII_Top_mdc_ch0_o                       (                               ),
        .QSGMII_Top_mdio_in_ch0_i                   (1'b0                           ),
        .QSGMII_Top_mdio_out_ch0_o                  (                               ),
        .QSGMII_Top_mdio_oen_ch0_o                  (                               ),
    //pcs config
        .QSGMII_Top_configuration_vector_ch0_i      (configuration_vector[0*15+:15] ),
        .QSGMII_Top_status_vector_ch0_o             (status_vector[0*11+:11]        ),
        .QSGMII_Top_link_partner_ability_base_ch0_o (                               ),

    //connected to an optical module to detect the presence of light
        .QSGMII_Top_signal_detect_ch1_i             (1'b1                           ),//1:presence; 0:not presence
    //config port
    //link_timer; multiplied by 32.768us
        .QSGMII_Top_sgmii_link_timer_ch1_i          (sgmii_link_timer[17:9]         ),
        .QSGMII_Top_sgmii_mode_ch1_i                (sgmii_mode[1]                  ),//0:mac mode; 1:phy mode
        .QSGMII_Top_repeater_mode_ch1_i             (1'b0                           ),

        .QSGMII_Top_rx_mac_clk_ch1_o                (rx_mac_clk[1]                  ),
        .QSGMII_Top_rx_mac_valid_ch1_o              (rx_mac_valid[1]                ),
        .QSGMII_Top_rx_mac_data_ch1_o               (rx_mac_data[1*8+:8]            ),
        .QSGMII_Top_rx_mac_last_ch1_o               (rx_mac_last[1]                 ),
        .QSGMII_Top_rx_mac_error_ch1_o              (rx_mac_error[1]                ),

        .QSGMII_Top_tx_mac_clk_ch1_o                (tx_mac_clk[1]                  ),
        .QSGMII_Top_tx_mac_valid_ch1_i              (tx_mac_valid[1]                ),
        .QSGMII_Top_tx_mac_data_ch1_i               (tx_mac_data[1*8+:8]            ),
        .QSGMII_Top_tx_mac_last_ch1_i               (tx_mac_last[1]                 ),
        .QSGMII_Top_tx_mac_error_ch1_i              (tx_mac_error[1]                ),
        .QSGMII_Top_tx_mac_ready_ch1_o              (tx_mac_ready[1]                ),
        .QSGMII_Top_tx_collision_ch1_o              (                               ),
        .QSGMII_Top_tx_retransmit_ch1_o             (                               ),

        .QSGMII_Top_rx_fcs_fwd_ena_ch1_i            (1'b0                           ),
        .QSGMII_Top_rx_jumbo_ena_ch1_i              (1'b0                           ),
        .QSGMII_Top_rx_statistics_valid_ch1_o       (rx_statistics_valid[1]         ),
        .QSGMII_Top_rx_statistics_vector_ch1_o      (rx_statistics_vector[1*27+:27] ),
        .QSGMII_Top_rx_pause_req_ch1_o              (rx_pause_req[1]                ),
        .QSGMII_Top_rx_pause_val_ch1_o              (rx_pause_val[1*16+:16]         ),

        .QSGMII_Top_tx_pause_req_ch1_i              (tx_pause_req[1]                ),
        .QSGMII_Top_tx_pause_val_ch1_i              (tx_pause_val[1*16+:16]         ),
        .QSGMII_Top_tx_pause_source_addr_ch1_i      (tx_pause_source_addr[1*48+:48] ),
        .QSGMII_Top_tx_ifg_delay_ena_ch1_i          (1'b0                           ),
        .QSGMII_Top_tx_ifg_delay_ch1_i              (8'd0                           ),
        .QSGMII_Top_tx_fcs_fwd_ena_ch1_i            (1'b0                           ),
        .QSGMII_Top_tx_statistics_valid_ch1_o       (tx_statistics_valid[1]         ),
        .QSGMII_Top_tx_statistics_vector_ch1_o      (                               ),
    //mdio
        .QSGMII_Top_miim_hs_clk_ch1_i               (1'b0                           ),
        .QSGMII_Top_miim_phyad_ch1_i                ({5{1'b0}}                      ),
        .QSGMII_Top_miim_regad_ch1_i                ({5{1'b0}}                      ),
        .QSGMII_Top_miim_wrdata_ch1_i               ({16{1'b0}}                     ),
        .QSGMII_Top_miim_wren_ch1_i                 (1'b0                           ),
        .QSGMII_Top_miim_rden_ch1_i                 (1'b0                           ),
        .QSGMII_Top_miim_rddata_ch1_o               (                               ),
        .QSGMII_Top_miim_rddata_valid_ch1_o         (                               ),
        .QSGMII_Top_miim_busy_ch1_o                 (                               ),
        .QSGMII_Top_mdc_ch1_o                       (                               ),
        .QSGMII_Top_mdio_in_ch1_i                   (1'b0                           ),
        .QSGMII_Top_mdio_out_ch1_o                  (                               ),
        .QSGMII_Top_mdio_oen_ch1_o                  (                               ),
    //pcs config
        .QSGMII_Top_configuration_vector_ch1_i      (configuration_vector[1*15+:15] ),
        .QSGMII_Top_status_vector_ch1_o             (status_vector[1*11+:11]        ),
        .QSGMII_Top_link_partner_ability_base_ch1_o (                               ),

    //connected to an optical module to detect the presence of light
        .QSGMII_Top_signal_detect_ch2_i             (1'b1                           ),//1:presence; 0:not presence
    //config port
    //link_timer; multiplied by 32.768us
        .QSGMII_Top_sgmii_link_timer_ch2_i          (sgmii_link_timer[26:18]        ),
        .QSGMII_Top_sgmii_mode_ch2_i                (sgmii_mode[2]                  ),//0:mac mode; 1:phy mode
        .QSGMII_Top_repeater_mode_ch2_i             (1'b0                           ),

        .QSGMII_Top_rx_mac_clk_ch2_o                (rx_mac_clk[2]                  ),
        .QSGMII_Top_rx_mac_valid_ch2_o              (rx_mac_valid[2]                ),
        .QSGMII_Top_rx_mac_data_ch2_o               (rx_mac_data[2*8+:8]            ),
        .QSGMII_Top_rx_mac_last_ch2_o               (rx_mac_last[2]                 ),
        .QSGMII_Top_rx_mac_error_ch2_o              (rx_mac_error[2]                ),

        .QSGMII_Top_tx_mac_clk_ch2_o                (tx_mac_clk[2]                  ),
        .QSGMII_Top_tx_mac_valid_ch2_i              (tx_mac_valid[2]                ),
        .QSGMII_Top_tx_mac_data_ch2_i               (tx_mac_data[2*8+:8]            ),
        .QSGMII_Top_tx_mac_last_ch2_i               (tx_mac_last[2]                 ),
        .QSGMII_Top_tx_mac_error_ch2_i              (tx_mac_error[2]                ),
        .QSGMII_Top_tx_mac_ready_ch2_o              (tx_mac_ready[2]                ),
        .QSGMII_Top_tx_collision_ch2_o              (                               ),
        .QSGMII_Top_tx_retransmit_ch2_o             (                               ),

        .QSGMII_Top_rx_fcs_fwd_ena_ch2_i            (1'b0                           ),
        .QSGMII_Top_rx_jumbo_ena_ch2_i              (1'b0                           ),
        .QSGMII_Top_rx_statistics_valid_ch2_o       (rx_statistics_valid[2]         ),
        .QSGMII_Top_rx_statistics_vector_ch2_o      (rx_statistics_vector[2*27+:27] ),
        .QSGMII_Top_rx_pause_req_ch2_o              (rx_pause_req[2]                ),
        .QSGMII_Top_rx_pause_val_ch2_o              (rx_pause_val[2*16+:16]         ),

        .QSGMII_Top_tx_pause_req_ch2_i              (tx_pause_req[2]                ),
        .QSGMII_Top_tx_pause_val_ch2_i              (tx_pause_val[2*16+:16]         ),
        .QSGMII_Top_tx_pause_source_addr_ch2_i      (tx_pause_source_addr[2*48+:48] ),
        .QSGMII_Top_tx_ifg_delay_ena_ch2_i          (1'b0                           ),
        .QSGMII_Top_tx_ifg_delay_ch2_i              (8'd0                           ),
        .QSGMII_Top_tx_fcs_fwd_ena_ch2_i            (1'b0                           ),
        .QSGMII_Top_tx_statistics_valid_ch2_o       (tx_statistics_valid[2]         ),
        .QSGMII_Top_tx_statistics_vector_ch2_o      (                               ),
    //mdio
        .QSGMII_Top_miim_hs_clk_ch2_i               (1'b0                           ),
        .QSGMII_Top_miim_phyad_ch2_i                ({5{1'b0}}                      ),
        .QSGMII_Top_miim_regad_ch2_i                ({5{1'b0}}                      ),
        .QSGMII_Top_miim_wrdata_ch2_i               ({16{1'b0}}                     ),
        .QSGMII_Top_miim_wren_ch2_i                 (1'b0                           ),
        .QSGMII_Top_miim_rden_ch2_i                 (1'b0                           ),
        .QSGMII_Top_miim_rddata_ch2_o               (                               ),
        .QSGMII_Top_miim_rddata_valid_ch2_o         (                               ),
        .QSGMII_Top_miim_busy_ch2_o                 (                               ),
        .QSGMII_Top_mdc_ch2_o                       (                               ),
        .QSGMII_Top_mdio_in_ch2_i                   (1'b0                           ),
        .QSGMII_Top_mdio_out_ch2_o                  (                               ),
        .QSGMII_Top_mdio_oen_ch2_o                  (                               ),
    //pcs config
        .QSGMII_Top_configuration_vector_ch2_i      (configuration_vector[2*15+:15] ),
        .QSGMII_Top_status_vector_ch2_o             (status_vector[2*11+:11]        ),
        .QSGMII_Top_link_partner_ability_base_ch2_o (                               ),

    //connected to an optical module to detect the presence of light
        .QSGMII_Top_signal_detect_ch3_i             (1'b1                           ),//1:presence; 0:not presence
    //config port
    //link_timer; multiplied by 32.768us
        .QSGMII_Top_sgmii_link_timer_ch3_i          (sgmii_link_timer[35:27]        ),
        .QSGMII_Top_sgmii_mode_ch3_i                (sgmii_mode[3]                  ),//0:mac mode; 1:phy mode
        .QSGMII_Top_repeater_mode_ch3_i             (1'b0                           ),

        .QSGMII_Top_rx_mac_clk_ch3_o                (rx_mac_clk[3]                  ),
        .QSGMII_Top_rx_mac_valid_ch3_o              (rx_mac_valid[3]                ),
        .QSGMII_Top_rx_mac_data_ch3_o               (rx_mac_data[3*8+:8]            ),
        .QSGMII_Top_rx_mac_last_ch3_o               (rx_mac_last[3]                 ),
        .QSGMII_Top_rx_mac_error_ch3_o              (rx_mac_error[3]                ),

        .QSGMII_Top_tx_mac_clk_ch3_o                (tx_mac_clk[3]                  ),
        .QSGMII_Top_tx_mac_valid_ch3_i              (tx_mac_valid[3]                ),
        .QSGMII_Top_tx_mac_data_ch3_i               (tx_mac_data[3*8+:8]            ),
        .QSGMII_Top_tx_mac_last_ch3_i               (tx_mac_last[3]                 ),
        .QSGMII_Top_tx_mac_error_ch3_i              (tx_mac_error[3]                ),
        .QSGMII_Top_tx_mac_ready_ch3_o              (tx_mac_ready[3]                ),
        .QSGMII_Top_tx_collision_ch3_o              (                               ),
        .QSGMII_Top_tx_retransmit_ch3_o             (                               ),

        .QSGMII_Top_rx_fcs_fwd_ena_ch3_i            (1'b0                           ),
        .QSGMII_Top_rx_jumbo_ena_ch3_i              (1'b0                           ),
        .QSGMII_Top_rx_statistics_valid_ch3_o       (rx_statistics_valid[3]         ),
        .QSGMII_Top_rx_statistics_vector_ch3_o      (rx_statistics_vector[3*27+:27] ),
        .QSGMII_Top_rx_pause_req_ch3_o              (rx_pause_req[3]                ),
        .QSGMII_Top_rx_pause_val_ch3_o              (rx_pause_val[3*16+:16]         ),

        .QSGMII_Top_tx_pause_req_ch3_i              (tx_pause_req[3]                ),
        .QSGMII_Top_tx_pause_val_ch3_i              (tx_pause_val[3*16+:16]         ),
        .QSGMII_Top_tx_pause_source_addr_ch3_i      (tx_pause_source_addr[3*48+:48] ),
        .QSGMII_Top_tx_ifg_delay_ena_ch3_i          (1'b0                           ),
        .QSGMII_Top_tx_ifg_delay_ch3_i              (8'd0                           ),
        .QSGMII_Top_tx_fcs_fwd_ena_ch3_i            (1'b0                           ),
        .QSGMII_Top_tx_statistics_valid_ch3_o       (tx_statistics_valid[3]         ),
        .QSGMII_Top_tx_statistics_vector_ch3_o      (                               ),
    //mdio
        .QSGMII_Top_miim_hs_clk_ch3_i               (1'b0                           ),
        .QSGMII_Top_miim_phyad_ch3_i                ({5{1'b0}}                      ),
        .QSGMII_Top_miim_regad_ch3_i                ({5{1'b0}}                      ),
        .QSGMII_Top_miim_wrdata_ch3_i               ({16{1'b0}}                     ),
        .QSGMII_Top_miim_wren_ch3_i                 (1'b0                           ),
        .QSGMII_Top_miim_rden_ch3_i                 (1'b0                           ),
        .QSGMII_Top_miim_rddata_ch3_o               (                               ),
        .QSGMII_Top_miim_rddata_valid_ch3_o         (                               ),
        .QSGMII_Top_miim_busy_ch3_o                 (                               ),
        .QSGMII_Top_mdc_ch3_o                       (                               ),
        .QSGMII_Top_mdio_in_ch3_i                   (1'b0                           ),
        .QSGMII_Top_mdio_out_ch3_o                  (                               ),
        .QSGMII_Top_mdio_oen_ch3_o                  (                               ),
    //pcs config
        .QSGMII_Top_configuration_vector_ch3_i      (configuration_vector[3*15+:15] ),
        .QSGMII_Top_status_vector_ch3_o             (status_vector[3*11+:11]        ),
        .QSGMII_Top_link_partner_ability_base_ch3_o (                               )
    );

Uart_to_Bus_Top u_Uart_to_Bus_Top(
        .rst_n_i                        (rstn_uart                      ),
        .clk_i                          (clk_uart                       ),
        .local0_wren_o                  (uart_wren                      ),
        .local0_addr_o                  (uart_addr                      ),
        .local0_rden_o                  (uart_rden                      ),
        .local0_wdat_o                  (uart_wdata                     ),
        .local0_rdat_i                  (uart_rdata                     ),
        .local0_rdat_vld_i              (uart_rdat_vld                  ),
        .local0_wdat_rdy_i              (uart_wdat_rdy                  ),
        .uart_rx_led_o                  (                               ),
        .uart_tx_led_o                  (                               ),
        .uart_rx_i                      (uart_rx                        ),
        .uart_tx_o                      (uart_tx                        )
    );

sysreg u_sysreg(                           
        .i_rst_n                        (rstn_uart                      ), 
        .i_clk                          (clk_uart                       ),

        .uart_wren                      (uart_wren                      ),
        .uart_addr                      (uart_addr                      ),
        .uart_rden                      (uart_rden                      ),
        .uart_wdata                     (uart_wdata                     ),
        .uart_rdata                     (uart_rdata                     ),
        .uart_rdat_vld                  (uart_rdat_vld                  ),
        .uart_wdat_rdy                  (uart_wdat_rdy                  ),

        .reg0x0000(reg0x0000),.reg0x0010(reg0x0010  ),.reg0x0020(reg0x0020  ),.reg0x0030(reg0x0030  ),
        .reg0x0001(reg0x0001),.reg0x0011(reg0x0011  ),.reg0x0021(reg0x0021  ),.reg0x0031(           ),
        .reg0x0002(reg0x0002),.reg0x0012(reg0x0012  ),.reg0x0022(reg0x0022  ),.reg0x0032(           ),
        .reg0x0003(         ),.reg0x0013(           ),.reg0x0023(           ),.reg0x0033(           ),

        .mon0x0080(mon0x0080),.mon0x0090(mon0x0090  ),.mon0x00A0(mon0x00A0  ),.mon0x00B0(mon0x00B0  ),
        .mon0x0081(mon0x0081),.mon0x0091(mon0x0091  ),.mon0x00A1(mon0x00A1  ),.mon0x00B1(mon0x00B1  ),
        .mon0x0082(32'd0    ),.mon0x0092(32'd0      ),.mon0x00A2(mon0x00A2  ),.mon0x00B2(mon0x00B2  ),
        .mon0x0083(32'd0    ),.mon0x0093(32'd0      ),.mon0x00A3(mon0x00A3  ),.mon0x00B3(mon0x00B3  )
    );

assign  mon0x0080[30:27] = 4'd0;    assign mon0x0080[22:19] = 4'd0; assign mon0x0080[14:11] = 4'd0; assign mon0x0080[6:3] = 4'd0;
assign  mon0x0081[31:27] = 5'd0;    assign mon0x0081[23:19] = 5'd0; assign mon0x0081[15:11] = 5'd0; assign mon0x0081[7:3] = 5'd0;
assign  mon0x0090[31:16] = 16'd0;
assign  mon0x0091[31:8]  = 24'd0;
assign  mon0x00A0 = tx_cnt[0*32+:32];
assign  mon0x00A1 = tx_cnt[1*32+:32];
assign  mon0x00A2 = tx_cnt[2*32+:32];
assign  mon0x00A3 = tx_cnt[3*32+:32];
assign  mon0x00B0 = rx_cnt[0*32+:32];
assign  mon0x00B1 = rx_cnt[1*32+:32];
assign  mon0x00B2 = rx_cnt[2*32+:32];
assign  mon0x00B3 = rx_cnt[3*32+:32];

assign  mon0x0090[15:0] = { ip_link_complete[3] ,ip_link_complete[2],ip_link_complete[1],ip_link_complete[0],
                            rx_mac_rst_n[3]     ,tx_mac_rst_n[3]    ,rx_mac_rst_n[2]    ,tx_mac_rst_n[2]    ,
                            rx_mac_rst_n[1]     ,tx_mac_rst_n[1]    ,rx_mac_rst_n[0]    ,tx_mac_rst_n[0]    ,
                            1'b1                ,rstn_ip            ,rstn_uart          ,pll_uart_lock      };

assign  sgmii_link_timer[8:0]   =   reg0x0021[8:0];     assign  sgmii_link_timer[17:9]  =   reg0x0021[24:16];   assign  sgmii_link_timer[26:18] =   reg0x0022[8:0];     assign  sgmii_link_timer[35:27] =   reg0x0022[24:16];

genvar i;
generate
    for(i=0;i<4;i=i+1) begin:test
        assign  main_reset[i]               =   reg0x0010[i];
        assign  an_enable[i]                =   reg0x0011[i*2+1];
        assign  an_restart[i]               =   reg0x0011[i*2];
        assign  disparity_checking_en[i]    =   reg0x0012[i];
        assign  sgmii_mode[i]               =   reg0x0020[i];
        assign  confi_vector[i*3+:3]        =   reg0x0020[(i*4+4)+:3];
        
        always @(posedge rx_mac_clk[i] or negedge rstn_ip) begin
            if(!rstn_ip) begin
                configuration_vector[i*15+:15]  <=  {3'b011,4'b0000,4'b1000,4'b0100};
            end else begin
                configuration_vector[i*15+0]            <=  main_reset[i];
                configuration_vector[i*15+2]            <=  an_enable[i];
                configuration_vector[i*15+4]            <=  an_restart[i];
                configuration_vector[i*15+1]            <=  configuration_vector[i*15+1];
                configuration_vector[i*15+3]            <=  configuration_vector[i*15+3];
                configuration_vector[i*15+7]            <=  disparity_checking_en[i];
                configuration_vector[i*15+6:i*15+5]     <=  configuration_vector[i*15+6:i*15+5];
                configuration_vector[i*15+10:i*15+8]    <=  configuration_vector[i*15+10:i*15+8];
                configuration_vector[i*15+13:i*15+11]   <=  confi_vector[i*3+:3];
                configuration_vector[i*15+14]           <=  configuration_vector[i*15+14];
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rstn_ip) begin
            if(!rstn_ip) begin
                ip_link_complete[i] <=  1'b0;
            end else if(an_enable[i]) begin
                ip_link_complete[i] <=   status_vector[i*11+2] && status_vector[i*11+1] && status_vector[i*11];
            end else begin
                ip_link_complete[i] <=   status_vector[i*11+1] && status_vector[i*11];    
            end
        end

        assign  mon0x0081[i*8+2:i*8] = status_vector[i*11+7:i*11+5];

        always @(posedge rx_mac_clk[i] or negedge rstn_ip) begin
            if(!rstn_ip) begin
                an_complete[i]      <=  1'b0;
                an_complete_d1[i]   <=  1'b0;
            end else begin
                an_complete[i]      <=  status_vector[i*11+2];
                an_complete_d1[i]   <=  an_complete[i];
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rstn_ip) begin
            if(!rstn_ip) begin
                ip_link_complete_cnt_rx[i*8+:8] <=  8'd0;
            end else if(ip_link_complete[i])begin
                if(ip_link_complete_cnt_rx[i*8+:8] == 8'h7f) begin
                    ip_link_complete_cnt_rx[i*8+:8] <=  ip_link_complete_cnt_rx[i*8+:8];
                end else begin
                    ip_link_complete_cnt_rx[i*8+:8] <=  ip_link_complete_cnt_rx[i*8+:8] + 8'd1;
                end
            end else begin
                ip_link_complete_cnt_rx[i*8+:8] <=  8'd0;
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rstn_ip) begin
            if(!rstn_ip) begin
                rx_mac_rst_n[i] <=  1'b0;
            end else if(ip_link_complete[i] && (ip_link_complete_cnt_rx[i*8+:8] == 8'h7f) ) begin
                rx_mac_rst_n[i] <=  1'b1;
            end else begin
                rx_mac_rst_n[i] <=  1'b0;  
            end
        end

        always @(posedge tx_mac_clk[i] or negedge rstn_ip) begin
            if(!rstn_ip) begin
                ip_link_complete_cnt_tx[i*8+:8] <=  8'd0;
            end else if(ip_link_complete[i])begin
                if(ip_link_complete_cnt_tx[i*8+:8] == 8'h7f) begin
                    ip_link_complete_cnt_tx[i*8+:8] <=  ip_link_complete_cnt_tx[i*8+:8];
                end else begin
                    ip_link_complete_cnt_tx[i*8+:8] <=  ip_link_complete_cnt_tx[i*8+:8] + 8'd1;
                end
            end else begin
                ip_link_complete_cnt_tx[i*8+:8] <=  8'd0;
            end
        end

        always @(posedge tx_mac_clk[i] or negedge rstn_ip) begin
            if(!rstn_ip) begin
                tx_mac_rst_n[i] <=  1'b0;
            end else if(ip_link_complete[i] && (ip_link_complete_cnt_tx[i*8+:8] == 8'h7f) ) begin
                tx_mac_rst_n[i] <=  1'b1;
            end else begin
                tx_mac_rst_n[i] <=  1'b0;  
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rstn_ip) begin
            if(~rstn_ip) begin
                auto_negotiation_complete[i]    <=  1'b0;
            end else if(reg0x0002[i]) begin
                auto_negotiation_complete[i]    <=  1'b0;
            end else if(auto_negotiation_complete[i]) begin
                auto_negotiation_complete[i]    <=  1'b1;
            end else begin
                if(an_complete[i] && ~an_complete_d1[i]) begin
                    auto_negotiation_complete[i]    <=  1'b1;
                end else begin
                    auto_negotiation_complete[i]    <=  1'b0;
                end
            end
        end
        
        assign  mon0x0080[i*8+7] = auto_negotiation_complete[i];

        always @(posedge rx_mac_clk[i] or negedge rx_mac_rst_n[i]) begin
            if(~rx_mac_rst_n[i]) begin
                rx_running_d1[i]    <=  1'b0;
            end else begin
                rx_running_d1[i]    <=  rx_running[i];
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rx_mac_rst_n[i]) begin
            if(~rx_mac_rst_n[i]) begin
                cycle_one_time[i]   <=  1'b0;
            end else begin
                cycle_one_time[i]   <=  reg0x0030[(i*2+4)];
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rx_mac_rst_n[i]) begin
            if(~rx_mac_rst_n[i]) begin
                cycle_all_time[i]   <=  1'b0;
            end else if(reg0x0030[(i*2+5)]) begin
                cycle_all_time[i]   <=  1'b1;
            end else begin
                if(cycle_all_time[i]) begin
                    if(rx_running[i] && ~rx_running_d1[i]) begin
                        cycle_all_time[i]   <=  1'b0;
                    end else begin
                        cycle_all_time[i]   <=  1'b1;
                    end
                end else begin
                    cycle_all_time[i]   <=  1'b0;
                end
            end
        end

        mac_tx_model u_mac_tx_model(
                .tx_mac_clk                 (tx_mac_clk[i]                  ),
                .tx_mac_rst_n               (tx_mac_rst_n[i]                ),

            //CONTROL PORT
                .i_cycle_one_time           (cycle_one_time[i]              ),
                .i_cycle_all_time           (cycle_all_time[i]              ),

                .i_tx_length_hold           (reg0x0000[i]                   ),
                .i_tx_no_pause              (reg0x0001[i]                   ),
                .i_tx_check_clear           (reg0x0002[i]                   ),

                .tx_running                 (tx_running[i]                  ),

            //IP TX PORT
                .tx_mac_valid               (tx_mac_valid[i]                ),
                .tx_mac_data                (tx_mac_data[i*8+:8]            ),
                .tx_mac_last                (tx_mac_last[i]                 ),
                .tx_mac_error               (tx_mac_error[i]                ),
                .tx_mac_ready               (tx_mac_ready[i]                ),

                .tx_pause_req               (tx_pause_req[i]                ),
                .tx_pause_val               (tx_pause_val[i*16+:16]         ),
                .tx_pause_source_addr       (tx_pause_source_addr[i*48+:48] ),
                .tx_statistics_valid        (tx_statistics_valid[i]         ),
                .tx_cnt_er                  (mon0x0091[i] 					),
                .tx_cnt                     (tx_cnt[i*32+:32]               )
            );
        
        mac_rx_model u_mac_rx_model(
                .rx_mac_clk                 (rx_mac_clk[i]                  ),
                .rx_mac_rst_n               (rx_mac_rst_n[i]                ),

            //CONTROL PORT
                .i_cycle_one_time           (cycle_one_time[i]              ),
                .i_cycle_all_time           (cycle_all_time[i]              ),

                .i_rx_length_hold           (reg0x0000[i]                   ),
                .i_rx_no_pause              (reg0x0001[i]                   ),
                .i_rx_check_clear           (reg0x0002[i]                   ),

                .rx_running                 (rx_running[i]                  ),

            //IP TX PORT
                .rx_mac_valid               (rx_mac_valid[i]                ),
                .rx_mac_data                (rx_mac_data[i*8+:8]            ),
                .rx_mac_last                (rx_mac_last[i]                 ),
                .rx_mac_error               (rx_mac_error[i]                ),

                .rx_statistics_valid        (rx_statistics_valid[i]         ),
                .rx_statistics_vector       (rx_statistics_vector[i*27+:27] ),
                .rx_pause_req               (rx_pause_req[i]                ),
                .rx_pause_val               (rx_pause_val[i*16+:16]         ),
        
            //CHECK RESULT
                .rx_data_error              (                               ),
                .rx_cnt_error               (                               ),
                .rx_statistics_error        (                               ),
                .rx_error_all_data          (                               ),

                .rx_data_error_pause        (                               ),
                .rx_length_error_pause      (                               ),
                .rx_statistics_error_pause  (                               ),
                .rx_pause_error             (                               ),
                .rx_error_all_pause         (                               ),

                .rx_error_all               (                               ),
                .rx_error_all_data_check    (mon0x0080[i*8]                 ),
                .rx_error_all_pause_check   (mon0x0080[i*8+1]               ),
                .rx_error_all_check         (mon0x0080[i*8+2]               ),
                .rx_cnt_er                  (mon0x0091[i+4]                 ),
                .rx_cnt                     (rx_cnt[i*32+:32]               )
            );
    end
endgenerate

assign  led_d1 = & ip_link_complete;
assign  led_d2 = (|tx_running) || (|rx_running);
assign  led_d3 = mon0x0080[0] || mon0x0080[8] || mon0x0080[16] || mon0x0080[24];
assign  led_d4 = mon0x0080[1] || mon0x0080[9] || mon0x0080[17] || mon0x0080[25];

endmodule

