`timescale 1ns/1ps

module tb;

localparam      RX_NUM              =   16'd4;

reg                 clk_in;
reg                 rstn;
reg                 serdes_pcs_tx_clk;
reg                 serdes_pcs_rx_clk;

wire    [43:0]      serdes_rxdata;
wire    [35:0]      serdes_txdata;

// //IP configuration_vector
reg     [1*4-1:0]   sgmii_mode;
reg     [9*4-1:0]   sgmii_link_timer;

reg     [1*4-1:0]   main_reset;
reg     [1*4-1:0]   an_enable;
reg     [1*4-1:0]   an_restart;
reg     [3*4-1:0]   confi_vector;
reg     [1*4-1:0]   disparity_checking_en;

reg     [1*4-1:0]   ip_link_complete;
reg     [1*4-1:0]   an_complete;
reg     [1*4-1:0]   an_complete_d1;
reg     [8*4-1:0]   ip_link_complete_cnt_tx;
reg     [1*4-1:0]   tx_mac_rst_n;
reg     [8*4-1:0]   ip_link_complete_cnt_rx;
reg     [1*4-1:0]   rx_mac_rst_n;

reg     [1*4-1:0]   cycle_one_time;
reg     [1*4-1:0]   cycle_all_time;

reg     [1*4-1:0]   length_hold;
reg     [1*4-1:0]   no_pause;
reg     [1*4-1:0]   check_clear;

//IP TX PORT
wire    [1*4-1:0]   tx_mac_clk/* synthesis syn_keep=1 */;
wire    [1*4-1:0]   tx_mac_valid;
wire    [8*4-1:0]   tx_mac_data;
wire    [1*4-1:0]   tx_mac_last;
wire    [1*4-1:0]   tx_mac_error;
wire    [1*4-1:0]   tx_mac_ready;

wire    [1*4-1:0]   tx_pause_req;
wire    [16*4-1:0]  tx_pause_val;
wire    [48*4-1:0]  tx_pause_source_addr;
wire    [1*4-1:0]   tx_statistics_valid;

//IP RX PORT
wire    [1*4-1:0]   rx_mac_clk/* synthesis syn_keep=1 */;
wire    [1*4-1:0]   rx_mac_valid;
wire    [8*4-1:0]   rx_mac_data;
wire    [1*4-1:0]   rx_mac_last;
wire    [1*4-1:0]   rx_mac_error;

wire    [1*4-1:0]   rx_statistics_valid;
wire    [27*4-1:0]  rx_statistics_vector;
wire    [1*4-1:0]   rx_pause_req;
wire    [16*4-1:0]  rx_pause_val;

//IP PORT
reg     [15*4-1:0]  configuration_vector;
wire    [11*4-1:0]  status_vector;

wire    [1*4-1:0]   tx_running;
wire    [1*4-1:0]   rx_running;
wire    [1*4-1:0]   error_all_check;

reg     [15:0]      cycle_cnt;

QSGMII_Top u_QSGMII_Top(
        .clk_in_i                       (clk_in                         ),
        .rstn_i                         (rstn                           ),

        .serdes_pcs_tx_clk_i            (serdes_pcs_tx_clk              ),
        .serdes_pcs_rx_clk_i            (serdes_pcs_rx_clk              ),
        .serdes_qpll1_ok_i              (1'b1                           ),
        .serdes_qpll0_ok_i              (1'b1                           ),
        .serdes_cpll_ok_i               (1'b1                           ),
        .serdes_pma_rx_lock_i           (1'b1                           ),
        .serdes_tx_fifo_afull_i         (1'b0                           ),
        .serdes_fabric_ctrl_o           (                               ),
        .serdes_fabric_ctrl_h_o         (                               ),
        .serdes_fabric_rstn_o           (                               ),
        .serdes_fabric_tx_clk_o         (                               ),
        .serdes_fabric_rx_clk_o         (                               ),
        .serdes_pcs_tx_rst_o            (                               ),
        .serdes_pcs_rx_rst_o            (                               ),
        .serdes_fabric_tx_vld_o         (                               ),
        .serdes_rxfifo_aempty_i         (1'b0                           ),
        .serdes_rxfifo_rd_en_o          (                               ),

        .serdes_rxdata_i                (serdes_rxdata                  ),
        .serdes_txdata_o                (serdes_txdata                  ),

    //connected to an optical module to detect the presence of light
        .signal_detect_ch0_i            (1'b1                           ),//1:presence; 0:not presence
    //config port
    //link_timer; multiplied by 32.768us
        .sgmii_link_timer_ch0_i         (sgmii_link_timer[8:0]          ),
        .sgmii_mode_ch0_i               (sgmii_mode[0]                  ),//0:mac mode; 1:phy mode
        .repeater_mode_ch0_i            (1'b0                           ),

        .rx_mac_clk_ch0_o               (rx_mac_clk[0]                  ),
        .rx_mac_valid_ch0_o             (rx_mac_valid[0]                ),
        .rx_mac_data_ch0_o              (rx_mac_data[0*8+:8]            ),
        .rx_mac_last_ch0_o              (rx_mac_last[0]                 ),
        .rx_mac_error_ch0_o             (rx_mac_error[0]                ),
        
        .tx_mac_clk_ch0_o               (tx_mac_clk[0]                  ),
        .tx_mac_valid_ch0_i             (tx_mac_valid[0]                ),
        .tx_mac_data_ch0_i              (tx_mac_data[0*8+:8]            ),
        .tx_mac_last_ch0_i              (tx_mac_last[0]                 ),
        .tx_mac_error_ch0_i             (tx_mac_error[0]                ),
        .tx_mac_ready_ch0_o             (tx_mac_ready[0]                ),
        .tx_collision_ch0_o             (                               ),
        .tx_retransmit_ch0_o            (                               ),

        .rx_fcs_fwd_ena_ch0_i           (1'b0                           ),
        .rx_jumbo_ena_ch0_i             (1'b0                           ),
        .rx_statistics_valid_ch0_o      (rx_statistics_valid[0]         ),
        .rx_statistics_vector_ch0_o     (rx_statistics_vector[0*27+:27] ),
        .rx_pause_req_ch0_o             (rx_pause_req[0]                ),
        .rx_pause_val_ch0_o             (rx_pause_val[0*16+:16]         ),

        .tx_pause_req_ch0_i             (tx_pause_req[0]                ),
        .tx_pause_val_ch0_i             (tx_pause_val[0*16+:16]         ),
        .tx_pause_source_addr_ch0_i     (tx_pause_source_addr[0*48+:48] ),
        .tx_ifg_delay_ena_ch0_i         (1'b0                           ),
        .tx_ifg_delay_ch0_i             (8'd0                           ),
        .tx_fcs_fwd_ena_ch0_i           (1'b0                           ),
        .tx_statistics_valid_ch0_o      (tx_statistics_valid[0]         ),
        .tx_statistics_vector_ch0_o     (                               ),
    //mdio
        .miim_hs_clk_ch0_i              (1'b0                           ),
        .miim_phyad_ch0_i               ({5{1'b0}}                      ),
        .miim_regad_ch0_i               ({5{1'b0}}                      ),
        .miim_wrdata_ch0_i              ({16{1'b0}}                     ),
        .miim_wren_ch0_i                (1'b0                           ),
        .miim_rden_ch0_i                (1'b0                           ),
        .miim_rddata_ch0_o              (                               ),
        .miim_rddata_valid_ch0_o        (                               ),
        .miim_busy_ch0_o                (                               ),
        .mdc_ch0_o                      (                               ),
        .mdio_in_ch0_i                  (1'b0                           ),
        .mdio_out_ch0_o                 (                               ),
        .mdio_oen_ch0_o                 (                               ),
    //pcs config
        .configuration_vector_ch0_i     (configuration_vector[0*15+:15] ),
        .status_vector_ch0_o            (status_vector[0*11+:11]        ),
        .link_partner_ability_base_ch0_o(                               ),

    //connected to an optical module to detect the presence of light
        .signal_detect_ch1_i            (1'b1                           ),//1:presence; 0:not presence
    //config port
    //link_timer; multiplied by 32.768us
        .sgmii_link_timer_ch1_i         (sgmii_link_timer[17:9]         ),
        .sgmii_mode_ch1_i               (sgmii_mode[1]                  ),//0:mac mode; 1:phy mode
        .repeater_mode_ch1_i            (1'b0                           ),

        .rx_mac_clk_ch1_o               (rx_mac_clk[1]                  ),
        .rx_mac_valid_ch1_o             (rx_mac_valid[1]                ),
        .rx_mac_data_ch1_o              (rx_mac_data[1*8+:8]            ),
        .rx_mac_last_ch1_o              (rx_mac_last[1]                 ),
        .rx_mac_error_ch1_o             (rx_mac_error[1]                ),

        .tx_mac_clk_ch1_o               (tx_mac_clk[1]                  ),
        .tx_mac_valid_ch1_i             (tx_mac_valid[1]                ),
        .tx_mac_data_ch1_i              (tx_mac_data[1*8+:8]            ),
        .tx_mac_last_ch1_i              (tx_mac_last[1]                 ),
        .tx_mac_error_ch1_i             (tx_mac_error[1]                ),
        .tx_mac_ready_ch1_o             (tx_mac_ready[1]                ),
        .tx_collision_ch1_o             (                               ),
        .tx_retransmit_ch1_o            (                               ),

        .rx_fcs_fwd_ena_ch1_i           (1'b0                           ),
        .rx_jumbo_ena_ch1_i             (1'b0                           ),
        .rx_statistics_valid_ch1_o      (rx_statistics_valid[1]         ),
        .rx_statistics_vector_ch1_o     (rx_statistics_vector[1*27+:27] ),
        .rx_pause_req_ch1_o             (rx_pause_req[1]                ),
        .rx_pause_val_ch1_o             (rx_pause_val[1*16+:16]         ),

        .tx_pause_req_ch1_i             (tx_pause_req[1]                ),
        .tx_pause_val_ch1_i             (tx_pause_val[1*16+:16]         ),
        .tx_pause_source_addr_ch1_i     (tx_pause_source_addr[1*48+:48] ),
        .tx_ifg_delay_ena_ch1_i         (1'b0                           ),
        .tx_ifg_delay_ch1_i             (8'd0                           ),
        .tx_fcs_fwd_ena_ch1_i           (1'b0                           ),
        .tx_statistics_valid_ch1_o      (tx_statistics_valid[1]         ),
        .tx_statistics_vector_ch1_o     (                               ),
    //mdio
        .miim_hs_clk_ch1_i              (1'b0                           ),
        .miim_phyad_ch1_i               ({5{1'b0}}                      ),
        .miim_regad_ch1_i               ({5{1'b0}}                      ),
        .miim_wrdata_ch1_i              ({16{1'b0}}                     ),
        .miim_wren_ch1_i                (1'b0                           ),
        .miim_rden_ch1_i                (1'b0                           ),
        .miim_rddata_ch1_o              (                               ),
        .miim_rddata_valid_ch1_o        (                               ),
        .miim_busy_ch1_o                (                               ),
        .mdc_ch1_o                      (                               ),
        .mdio_in_ch1_i                  (1'b0                           ),
        .mdio_out_ch1_o                 (                               ),
        .mdio_oen_ch1_o                 (                               ),
    //pcs config
        .configuration_vector_ch1_i     (configuration_vector[1*15+:15] ),
        .status_vector_ch1_o            (status_vector[1*11+:11]        ),
        .link_partner_ability_base_ch1_o(                               ),

    //connected to an optical module to detect the presence of light
        .signal_detect_ch2_i            (1'b1                           ),//1:presence; 0:not presence
    //config port
    //link_timer; multiplied by 32.768us
        .sgmii_link_timer_ch2_i         (sgmii_link_timer[26:18]        ),
        .sgmii_mode_ch2_i               (sgmii_mode[2]                  ),//0:mac mode; 1:phy mode
        .repeater_mode_ch2_i            (1'b0                           ),

        .rx_mac_clk_ch2_o               (rx_mac_clk[2]                  ),
        .rx_mac_valid_ch2_o             (rx_mac_valid[2]                ),
        .rx_mac_data_ch2_o              (rx_mac_data[2*8+:8]            ),
        .rx_mac_last_ch2_o              (rx_mac_last[2]                 ),
        .rx_mac_error_ch2_o             (rx_mac_error[2]                ),

        .tx_mac_clk_ch2_o               (tx_mac_clk[2]                  ),
        .tx_mac_valid_ch2_i             (tx_mac_valid[2]                ),
        .tx_mac_data_ch2_i              (tx_mac_data[2*8+:8]            ),
        .tx_mac_last_ch2_i              (tx_mac_last[2]                 ),
        .tx_mac_error_ch2_i             (tx_mac_error[2]                ),
        .tx_mac_ready_ch2_o             (tx_mac_ready[2]                ),
        .tx_collision_ch2_o             (                               ),
        .tx_retransmit_ch2_o            (                               ),

        .rx_fcs_fwd_ena_ch2_i           (1'b0                           ),
        .rx_jumbo_ena_ch2_i             (1'b0                           ),
        .rx_statistics_valid_ch2_o      (rx_statistics_valid[2]         ),
        .rx_statistics_vector_ch2_o     (rx_statistics_vector[2*27+:27] ),
        .rx_pause_req_ch2_o             (rx_pause_req[2]                ),
        .rx_pause_val_ch2_o             (rx_pause_val[2*16+:16]         ),

        .tx_pause_req_ch2_i             (tx_pause_req[2]                ),
        .tx_pause_val_ch2_i             (tx_pause_val[2*16+:16]         ),
        .tx_pause_source_addr_ch2_i     (tx_pause_source_addr[2*48+:48] ),
        .tx_ifg_delay_ena_ch2_i         (1'b0                           ),
        .tx_ifg_delay_ch2_i             (8'd0                           ),
        .tx_fcs_fwd_ena_ch2_i           (1'b0                           ),
        .tx_statistics_valid_ch2_o      (tx_statistics_valid[2]         ),
        .tx_statistics_vector_ch2_o     (                               ),
    //mdio
        .miim_hs_clk_ch2_i              (1'b0                           ),
        .miim_phyad_ch2_i               ({5{1'b0}}                      ),
        .miim_regad_ch2_i               ({5{1'b0}}                      ),
        .miim_wrdata_ch2_i              ({16{1'b0}}                     ),
        .miim_wren_ch2_i                (1'b0                           ),
        .miim_rden_ch2_i                (1'b0                           ),
        .miim_rddata_ch2_o              (                               ),
        .miim_rddata_valid_ch2_o        (                               ),
        .miim_busy_ch2_o                (                               ),
        .mdc_ch2_o                      (                               ),
        .mdio_in_ch2_i                  (1'b0                           ),
        .mdio_out_ch2_o                 (                               ),
        .mdio_oen_ch2_o                 (                               ),
    //pcs config
        .configuration_vector_ch2_i     (configuration_vector[2*15+:15] ),
        .status_vector_ch2_o            (status_vector[2*11+:11]        ),
        .link_partner_ability_base_ch2_o(                               ),

    //connected to an optical module to detect the presence of light
        .signal_detect_ch3_i            (1'b1                           ),//1:presence; 0:not presence
    //config port
    //link_timer; multiplied by 32.768us
        .sgmii_link_timer_ch3_i         (sgmii_link_timer[35:27]        ),
        .sgmii_mode_ch3_i               (sgmii_mode[3]                  ),//0:mac mode; 1:phy mode
        .repeater_mode_ch3_i            (1'b0                           ),

        .rx_mac_clk_ch3_o               (rx_mac_clk[3]                  ),
        .rx_mac_valid_ch3_o             (rx_mac_valid[3]                ),
        .rx_mac_data_ch3_o              (rx_mac_data[3*8+:8]            ),
        .rx_mac_last_ch3_o              (rx_mac_last[3]                 ),
        .rx_mac_error_ch3_o             (rx_mac_error[3]                ),

        .tx_mac_clk_ch3_o               (tx_mac_clk[3]                  ),
        .tx_mac_valid_ch3_i             (tx_mac_valid[3]                ),
        .tx_mac_data_ch3_i              (tx_mac_data[3*8+:8]            ),
        .tx_mac_last_ch3_i              (tx_mac_last[3]                 ),
        .tx_mac_error_ch3_i             (tx_mac_error[3]                ),
        .tx_mac_ready_ch3_o             (tx_mac_ready[3]                ),
        .tx_collision_ch3_o             (                               ),
        .tx_retransmit_ch3_o            (                               ),

        .rx_fcs_fwd_ena_ch3_i           (1'b0                           ),
        .rx_jumbo_ena_ch3_i             (1'b0                           ),
        .rx_statistics_valid_ch3_o      (rx_statistics_valid[3]         ),
        .rx_statistics_vector_ch3_o     (rx_statistics_vector[3*27+:27] ),
        .rx_pause_req_ch3_o             (rx_pause_req[3]                ),
        .rx_pause_val_ch3_o             (rx_pause_val[3*16+:16]         ),

        .tx_pause_req_ch3_i             (tx_pause_req[3]                ),
        .tx_pause_val_ch3_i             (tx_pause_val[3*16+:16]         ),
        .tx_pause_source_addr_ch3_i     (tx_pause_source_addr[3*48+:48] ),
        .tx_ifg_delay_ena_ch3_i         (1'b0                           ),
        .tx_ifg_delay_ch3_i             (8'd0                           ),
        .tx_fcs_fwd_ena_ch3_i           (1'b0                           ),
        .tx_statistics_valid_ch3_o      (tx_statistics_valid[3]         ),
        .tx_statistics_vector_ch3_o     (                               ),
    //mdio
        .miim_hs_clk_ch3_i              (1'b0                           ),
        .miim_phyad_ch3_i               ({5{1'b0}}                      ),
        .miim_regad_ch3_i               ({5{1'b0}}                      ),
        .miim_wrdata_ch3_i              ({16{1'b0}}                     ),
        .miim_wren_ch3_i                (1'b0                           ),
        .miim_rden_ch3_i                (1'b0                           ),
        .miim_rddata_ch3_o              (                               ),
        .miim_rddata_valid_ch3_o        (                               ),
        .miim_busy_ch3_o                (                               ),
        .mdc_ch3_o                      (                               ),
        .mdio_in_ch3_i                  (1'b0                           ),
        .mdio_out_ch3_o                 (                               ),
        .mdio_oen_ch3_o                 (                               ),
    //pcs config
        .configuration_vector_ch3_i     (configuration_vector[3*15+:15] ),
        .status_vector_ch3_o            (status_vector[3*11+:11]        ),
        .link_partner_ability_base_ch3_o(                               )
    );

assign  serdes_rxdata = {2'b00,serdes_txdata[35:27],2'b00,serdes_txdata[26:18],2'b00,serdes_txdata[17:9],2'b00,serdes_txdata[8:0]};

genvar i;
generate
    for(i=0;i<4;i=i+1) begin:test
        
        always @(posedge rx_mac_clk[i] or negedge rstn) begin
            if(!rstn) begin
                configuration_vector[i*15+:15]  <=  {3'b011,4'b0000,4'b1000,4'b0100};
            end else begin
                configuration_vector[i*15+0]            <=  main_reset[i];
                configuration_vector[i*15+2]            <=  an_enable[i];
                configuration_vector[i*15+4]            <=  an_restart[i];
                configuration_vector[i*15+1]            <=  configuration_vector[i*15+1];
                configuration_vector[i*15+3]            <=  configuration_vector[i*15+3];
                configuration_vector[i*15+7]            <=  disparity_checking_en[i];
                configuration_vector[i*15+6:i*15+5]     <=  configuration_vector[i*15+6:i*15+5];
                configuration_vector[i*15+10:i*15+8]    <=  configuration_vector[i*15+10:i*15+8];
                configuration_vector[i*15+13:i*15+11]   <=  confi_vector[i*3+:3];
                configuration_vector[i*15+14]           <=  configuration_vector[i*15+14];
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rstn) begin
            if(!rstn) begin
                ip_link_complete[i] <=  1'b0;
            end else if(an_enable[i]) begin
                ip_link_complete[i] <=   status_vector[i*11+2] && status_vector[i*11+1] && status_vector[i*11];
            end else begin
                ip_link_complete[i] <=   status_vector[i*11+1] && status_vector[i*11];    
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rstn) begin
            if(!rstn) begin
                an_complete[i]      <=  1'b0;
                an_complete_d1[i]   <=  1'b0;
            end else begin
                an_complete[i]      <=  status_vector[i*11+2];
                an_complete_d1[i]   <=  an_complete[i];
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rstn) begin
            if(!rstn) begin
                ip_link_complete_cnt_rx[i*8+:8] <=  8'd0;
            end else if(ip_link_complete[i])begin
                if(ip_link_complete_cnt_rx[i*8+:8] == 8'h7f) begin
                    ip_link_complete_cnt_rx[i*8+:8] <=  ip_link_complete_cnt_rx[i*8+:8];
                end else begin
                    ip_link_complete_cnt_rx[i*8+:8] <=  ip_link_complete_cnt_rx[i*8+:8] + 8'd1;
                end
            end else begin
                ip_link_complete_cnt_rx[i*8+:8] <=  8'd0;
            end
        end

        always @(posedge rx_mac_clk[i] or negedge rstn) begin
            if(!rstn) begin
                rx_mac_rst_n[i] <=  1'b0;
            end else if(ip_link_complete[i] && (ip_link_complete_cnt_rx[i*8+:8] == 8'h7f) ) begin
                rx_mac_rst_n[i] <=  1'b1;
            end else begin
                rx_mac_rst_n[i] <=  1'b0;  
            end
        end

        always @(posedge tx_mac_clk[i] or negedge rstn) begin
            if(!rstn) begin
                ip_link_complete_cnt_tx[i*8+:8] <=  8'd0;
            end else if(ip_link_complete[i])begin
                if(ip_link_complete_cnt_tx[i*8+:8] == 8'h7f) begin
                    ip_link_complete_cnt_tx[i*8+:8] <=  ip_link_complete_cnt_tx[i*8+:8];
                end else begin
                    ip_link_complete_cnt_tx[i*8+:8] <=  ip_link_complete_cnt_tx[i*8+:8] + 8'd1;
                end
            end else begin
                ip_link_complete_cnt_tx[i*8+:8] <=  8'd0;
            end
        end

        always @(posedge tx_mac_clk[i] or negedge rstn) begin
            if(!rstn) begin
                tx_mac_rst_n[i] <=  1'b0;
            end else if(ip_link_complete[i] && (ip_link_complete_cnt_tx[i*8+:8] == 8'h7f) ) begin
                tx_mac_rst_n[i] <=  1'b1;
            end else begin
                tx_mac_rst_n[i] <=  1'b0;  
            end
        end

        mac_tx_model u_mac_tx_model(
                .tx_mac_clk                 (tx_mac_clk[i]                  ),
                .tx_mac_rst_n               (tx_mac_rst_n[i]                ),

            //CONTROL PORT
                .i_cycle_one_time           (cycle_one_time[i]              ),
                .i_cycle_all_time           (cycle_all_time[i]              ),

                .i_tx_length_hold           (length_hold[i]                 ),
                .i_tx_no_pause              (no_pause[i]                    ),
                .i_tx_check_clear           (check_clear[i]                 ),

                .tx_running                 (tx_running[i]                  ),

            //IP TX PORT
                .tx_mac_valid               (tx_mac_valid[i]                ),
                .tx_mac_data                (tx_mac_data[i*8+:8]            ),
                .tx_mac_last                (tx_mac_last[i]                 ),
                .tx_mac_error               (tx_mac_error[i]                ),
                .tx_mac_ready               (tx_mac_ready[i]                ),

                .tx_pause_req               (tx_pause_req[i]                ),
                .tx_pause_val               (tx_pause_val[i*16+:16]         ),
                .tx_pause_source_addr       (tx_pause_source_addr[i*48+:48] ),
                .tx_statistics_valid        (tx_statistics_valid[i]         ),
                .tx_cnt_er                  (                               ),
                .tx_cnt                     (                               )
            );
        
        mac_rx_model u_mac_rx_model(
                .rx_mac_clk                 (rx_mac_clk[i]                  ),
                .rx_mac_rst_n               (rx_mac_rst_n[i]                ),

            //CONTROL PORT
                .i_cycle_one_time           (cycle_one_time[i]              ),
                .i_cycle_all_time           (cycle_all_time[i]              ),

                .i_rx_length_hold           (length_hold[i]                 ),
                .i_rx_no_pause              (no_pause[i]                    ),
                .i_rx_check_clear           (check_clear[i]                 ),

                .rx_running                 (rx_running[i]                  ),

            //IP TX PORT
                .rx_mac_valid               (rx_mac_valid[i]                ),
                .rx_mac_data                (rx_mac_data[i*8+:8]            ),
                .rx_mac_last                (rx_mac_last[i]                 ),
                .rx_mac_error               (rx_mac_error[i]                ),

                .rx_statistics_valid        (rx_statistics_valid[i]         ),
                .rx_statistics_vector       (rx_statistics_vector[i*27+:27] ),
                .rx_pause_req               (rx_pause_req[i]                ),
                .rx_pause_val               (rx_pause_val[i*16+:16]         ),
        
            //CHECK RESULT
                .rx_data_error              (                               ),
                .rx_cnt_error               (                               ),
                .rx_statistics_error        (                               ),
                .rx_error_all_data          (                               ),

                .rx_data_error_pause        (                               ),
                .rx_length_error_pause      (                               ),
                .rx_statistics_error_pause  (                               ),
                .rx_pause_error             (                               ),
                .rx_error_all_pause         (                               ),

                .rx_error_all               (                               ),
                .rx_error_all_data_check    (                               ),
                .rx_error_all_pause_check   (                               ),
                .rx_error_all_check         (error_all_check[i]             ),
                .rx_cnt_er                  (                               ),
                .rx_cnt                     (                               )
            );
    end
endgenerate

initial begin
    clk_in                      =   0;
    forever begin
        #10;
        clk_in                  =   ~clk_in;
    end
end

initial begin
    serdes_pcs_tx_clk           =   0;
    forever begin
        #4;
        serdes_pcs_tx_clk       =   ~serdes_pcs_tx_clk;
    end
end

initial begin
    serdes_pcs_rx_clk           =   0;
    forever begin
        #4;
        serdes_pcs_rx_clk       =   ~serdes_pcs_rx_clk;
    end
end

task fd_one_time;
    begin
        cycle_one_time          =   {4{1'b1}};
        repeat(10) @(posedge serdes_pcs_rx_clk);
        cycle_one_time          =   {4{1'b0}};
        repeat(10) @(posedge serdes_pcs_rx_clk);
        wait( (tx_running == 4'd0) && (rx_running == 4'd0) );
    end
endtask

task fd_all_time;
    begin
        cycle_all_time          =   {4{1'b1}};
        repeat(10) @(posedge serdes_pcs_rx_clk);

        while(cycle_cnt < RX_NUM) begin
            @(posedge serdes_pcs_rx_clk);
            if (~rx_running[0]) begin
                cycle_cnt       <=  cycle_cnt + 16'd1;
            end else begin
                cycle_cnt       <=  cycle_cnt;
            end
        end

        @(posedge serdes_pcs_rx_clk);
        cycle_cnt               <=  16'd0;

        wait(~rx_running[3]);
        cycle_all_time          =   4'h7;
        wait(~rx_running[2]);
        cycle_all_time          =   4'h3;
        wait(~rx_running[1]);
        cycle_all_time          =   4'h1;
        wait(~rx_running[0]);
        cycle_all_time          =   4'h0;

        repeat(10) @(posedge serdes_pcs_rx_clk);
        wait( (tx_running == 4'd0) && (rx_running == 4'd0) );
    end
endtask

task functional_testing;
    begin
        wait( (rx_mac_rst_n == 4'hF) && (tx_mac_rst_n == 4'hF) );
        repeat(10) @(posedge serdes_pcs_rx_clk);
    
    //检测传输正常帧 负载在变化
        $display("--------------------NUM1    test  --- start transmitting normal frames : %0tus-------------------",$time);
        fd_one_time;
        $display("--------------------NUM1    test  --- test transmitting normal frames correctly and stop the test : %0tus-------------------",$time);
    
    //检测传输Pause帧
        $display("--------------------NUM2    test  --- start transmitting pause frames : %0tus-------------------",$time);
            fd_one_time;
        $display("--------------------NUM2    test  --- test transmitting pause frames correctly and stop the test : %0tus-------------------",$time);

    //检测满载下传输
        $display("--------------------NUM3    test  --- start testing fully loaded transmission : %0tus-------------------",$time);
        //1500负载
        no_pause                =   {4{1'b1}};
        length_hold             =   {4{1'b1}};
        repeat(10) @(posedge serdes_pcs_rx_clk);
        fd_all_time;
    
        no_pause                =   {4{1'b0}};
        length_hold             =   {4{1'b0}};
        repeat(10) @(posedge serdes_pcs_rx_clk);
        $display("--------------------NUM3    test  --- test fully loaded transmission correctly and stop the test : %0tus-------------------",$time);

    //其他功能:重新自协商等
        $display("--------------------NUM4    test  --- start testing other features : %0tus-------------------",$time);
        //main_reset
        main_reset              =   4'h1;
        repeat(40) @(posedge serdes_pcs_rx_clk);
        main_reset              =   {4{1'b0}};
        repeat(40) @(posedge serdes_pcs_rx_clk);
    
        wait( (rx_mac_rst_n == 4'hF) && (tx_mac_rst_n == 4'hF) );
        repeat(10) @(posedge serdes_pcs_rx_clk);
        fd_one_time;

        main_reset              =   4'h2;
        repeat(40) @(posedge serdes_pcs_rx_clk);
        main_reset              =   {4{1'b0}};
        repeat(40) @(posedge serdes_pcs_rx_clk);
    
        wait( (rx_mac_rst_n == 4'hF) && (tx_mac_rst_n == 4'hF) );
        repeat(10) @(posedge serdes_pcs_rx_clk);
        fd_one_time;

        main_reset              =   4'hC;
        repeat(40) @(posedge serdes_pcs_rx_clk);
        main_reset              =   {4{1'b0}};
        repeat(40) @(posedge serdes_pcs_rx_clk);
    
        wait( (rx_mac_rst_n == 4'hF) && (tx_mac_rst_n == 4'hF) );
        repeat(10) @(posedge serdes_pcs_rx_clk);
        fd_one_time;
    
        //an_restart
        an_restart              =   4'h1;
        repeat(40) @(posedge serdes_pcs_rx_clk);
        an_restart              =   {4{1'b0}};
        repeat(40) @(posedge serdes_pcs_rx_clk);

        wait( (rx_mac_rst_n == 4'hF) && (tx_mac_rst_n == 4'hF) );
        repeat(10) @(posedge serdes_pcs_rx_clk);
        fd_one_time;

        an_restart              =   4'h2;
        repeat(40) @(posedge serdes_pcs_rx_clk);
        an_restart              =   {4{1'b0}};
        repeat(40) @(posedge serdes_pcs_rx_clk);

        wait( (rx_mac_rst_n == 4'hF) && (tx_mac_rst_n == 4'hF) );
        repeat(10) @(posedge serdes_pcs_rx_clk);
        fd_one_time;

        an_restart              =   4'hC;
        repeat(40) @(posedge serdes_pcs_rx_clk);
        an_restart              =   {4{1'b0}};
        repeat(40) @(posedge serdes_pcs_rx_clk);

        wait( (rx_mac_rst_n == 4'hF) && (tx_mac_rst_n == 4'hF) );
        repeat(10) @(posedge serdes_pcs_rx_clk);
        fd_one_time;

        $display("--------------------NUM4    test  --- test other features correctly and stop the test : %0tus-------------------",$time);

    end
endtask

initial begin
    wait(error_all_check != 4'd0);
    $display("--------------%0tus:transceiver is incorrect-------------------",$time);
    $finish;
end

initial begin
        rstn                    =   1'b0;

        an_restart              =   {4{1'b0}};
        an_enable               =   {4{1'b1}};
        main_reset              =   {4{1'b0}};
        disparity_checking_en   =   {4{1'b1}};

        sgmii_mode              =   {4{1'b1}};
        sgmii_link_timer        =   {4{9'd3}};
        confi_vector            =   {3'b110,3'b100,3'b101,3'b110};

        length_hold             =   {4{1'b0}};
        no_pause                =   {4{1'b0}};
        check_clear             =   {4{1'b0}};

        cycle_one_time          =   {4{1'b0}};
        cycle_all_time          =   {4{1'b0}};

        cycle_cnt               =   16'd0;

        #100;
        rstn                    =   1'b1;
        
        wait(ip_link_complete == 4'hF);
        $display("--------------Auto-Negotiation is complete : %0tus-------------------",$time);

        wait( (rx_mac_rst_n == 4'hF) && (tx_mac_rst_n == 4'hF) );
        repeat(10) @(posedge serdes_pcs_rx_clk);
        $display("--------------MAC Layer Tests-------------------");

//检测全双工模式下的通信
        $display("----------------full duplex tests --- start testing full duplex : %0tus-------------------",$time);

    //Port0:1000M  Port1:100M  Port2:10M  Port3:1000M
        $display("------------------Port0:1000M  Port1:100M  Port2:10M  Port3:1000M-------------------");
        functional_testing;

        #1000;
        $finish;
end

initial begin
    $fsdbDumpfile("tb.fsdb");
    $fsdbDumpvars;
end

endmodule